<?php

/**
 * @license LGPLv3, http://opensource.org/licenses/LGPL-3.0
 * @copyright Aimeos (aimeos.org), 2014
 */


namespace Aimeos\Composer;

use Composer\Package\PackageInterface;
use Composer\Installer\LibraryInstaller;


/**
 * Custom composer installer for Aimeos extensions
 */
class ExtensionInstaller extends LibraryInstaller
{
	/**
	 * {@inheritDoc}
	 */
	public function getInstallPath( PackageInterface $package )
	{
		if( substr( $package->getPrettyName(), 0, 7 ) !== 'aimeos/' )
		{
			throw new \InvalidArgumentException(
				'Unable to install extension. Aimeos extension names should always start with "aimeos/"'
			);
		}

		$extname = substr( $package->getPrettyName(), 7 );
		$extra = $this->composer->getPackage()->getExtra();

		if( isset( $extra['ext-path'] ) ) {
			return $extra['ext-path'] . DIRECTORY_SEPARATOR . $extname;
		}

		return 'ext' . DIRECTORY_SEPARATOR . $extname;
	}


	/**
	 * {@inheritDoc}
	 */
	public function supports( $packageType )
	{
		return $packageType === 'aimeos-extension';
	}
}
